/** \file oid_std.c
    \brief Standard Object Identifiers: Generated from ./oid.csv */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <yaz/oid_db.h>

YAZ_EXPORT const Odr_oid yaz_oid_transyn_ber[] = {2,1,1,-1};
YAZ_EXPORT const Odr_oid yaz_oid_transyn_iso2709[] = {1,0,2709,1,1,-1};
YAZ_EXPORT const Odr_oid yaz_oid_general_isoill_1[] = {1,0,10161,2,1,-1};
YAZ_EXPORT const Odr_oid yaz_oid_absyn_z_apdu[] = {2,1,-1};
YAZ_EXPORT const Odr_oid yaz_oid_appctx_z_basic[] = {1,1,-1};
YAZ_EXPORT const Odr_oid yaz_oid_attset_bib_1[] = {Z3950_PREFIX,3,1,-1};
YAZ_EXPORT const Odr_oid yaz_oid_attset_exp_1[] = {Z3950_PREFIX,3,2,-1};
YAZ_EXPORT const Odr_oid yaz_oid_attset_ext_1[] = {Z3950_PREFIX,3,3,-1};
YAZ_EXPORT const Odr_oid yaz_oid_attset_ccl_1[] = {Z3950_PREFIX,3,4,-1};
YAZ_EXPORT const Odr_oid yaz_oid_attset_gils[] = {Z3950_PREFIX,3,5,-1};
YAZ_EXPORT const Odr_oid yaz_oid_attset_gils_attset[] = {Z3950_PREFIX,3,5,-1};
YAZ_EXPORT const Odr_oid yaz_oid_attset_stas_attset[] = {Z3950_PREFIX,3,6,-1};
YAZ_EXPORT const Odr_oid yaz_oid_attset_collections_attset[] = {Z3950_PREFIX,3,7,-1};
YAZ_EXPORT const Odr_oid yaz_oid_attset_cimi_attset[] = {Z3950_PREFIX,3,8,-1};
YAZ_EXPORT const Odr_oid yaz_oid_attset_geo_attset[] = {Z3950_PREFIX,3,9,-1};
YAZ_EXPORT const Odr_oid yaz_oid_attset_zbig[] = {Z3950_PREFIX,3,10,-1};
YAZ_EXPORT const Odr_oid yaz_oid_attset_util[] = {Z3950_PREFIX,3,11,-1};
YAZ_EXPORT const Odr_oid yaz_oid_attset_xd_1[] = {Z3950_PREFIX,3,12,-1};
YAZ_EXPORT const Odr_oid yaz_oid_attset_zthes[] = {Z3950_PREFIX,3,13,-1};
YAZ_EXPORT const Odr_oid yaz_oid_attset_fin_1[] = {Z3950_PREFIX,3,14,-1};
YAZ_EXPORT const Odr_oid yaz_oid_attset_dan_1[] = {Z3950_PREFIX,3,15,-1};
YAZ_EXPORT const Odr_oid yaz_oid_attset_holdings[] = {Z3950_PREFIX,3,16,-1};
YAZ_EXPORT const Odr_oid yaz_oid_attset_marc[] = {Z3950_PREFIX,3,17,-1};
YAZ_EXPORT const Odr_oid yaz_oid_attset_bib_2[] = {Z3950_PREFIX,3,18,-1};
YAZ_EXPORT const Odr_oid yaz_oid_attset_zeerex[] = {Z3950_PREFIX,3,19,-1};
YAZ_EXPORT const Odr_oid yaz_oid_attset_thesaurus_attset[] = {Z3950_PREFIX,3,1000,81,1,-1};
YAZ_EXPORT const Odr_oid yaz_oid_attset_idxpath[] = {Z3950_PREFIX,3,1000,81,2,-1};
YAZ_EXPORT const Odr_oid yaz_oid_attset_extlite[] = {Z3950_PREFIX,3,1000,81,3,-1};
YAZ_EXPORT const Odr_oid yaz_oid_diagset_bib_1[] = {Z3950_PREFIX,4,1,-1};
YAZ_EXPORT const Odr_oid yaz_oid_diagset_diag_1[] = {Z3950_PREFIX,4,2,-1};
YAZ_EXPORT const Odr_oid yaz_oid_diagset_diag_es[] = {Z3950_PREFIX,4,3,-1};
YAZ_EXPORT const Odr_oid yaz_oid_diagset_diag_general[] = {Z3950_PREFIX,4,3,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_unimarc[] = {Z3950_PREFIX,5,1,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_intermarc[] = {Z3950_PREFIX,5,2,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_ccf[] = {Z3950_PREFIX,5,3,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_usmarc[] = {Z3950_PREFIX,5,10,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_marc21[] = {Z3950_PREFIX,5,10,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_ukmarc[] = {Z3950_PREFIX,5,11,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_normarc[] = {Z3950_PREFIX,5,12,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_librismarc[] = {Z3950_PREFIX,5,13,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_danmarc[] = {Z3950_PREFIX,5,14,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_finmarc[] = {Z3950_PREFIX,5,15,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_mab[] = {Z3950_PREFIX,5,16,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_canmarc[] = {Z3950_PREFIX,5,17,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_sbn[] = {Z3950_PREFIX,5,18,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_picamarc[] = {Z3950_PREFIX,5,19,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_ausmarc[] = {Z3950_PREFIX,5,20,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_ibermarc[] = {Z3950_PREFIX,5,21,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_carmarc[] = {Z3950_PREFIX,5,22,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_malmarc[] = {Z3950_PREFIX,5,23,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_jpmarc[] = {Z3950_PREFIX,5,24,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_swemarc[] = {Z3950_PREFIX,5,25,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_siglemarc[] = {Z3950_PREFIX,5,26,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_isdsmarc[] = {Z3950_PREFIX,5,27,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_rusmarc[] = {Z3950_PREFIX,5,28,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_hunmarc[] = {Z3950_PREFIX,5,29,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_nacsis_catp[] = {Z3950_PREFIX,5,30,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_finmarc2000[] = {Z3950_PREFIX,5,31,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_marc21_fin[] = {Z3950_PREFIX,5,32,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_explain[] = {Z3950_PREFIX,5,100,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_sutrs[] = {Z3950_PREFIX,5,101,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_opac[] = {Z3950_PREFIX,5,102,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_summary[] = {Z3950_PREFIX,5,103,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_grs_0[] = {Z3950_PREFIX,5,104,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_grs_1[] = {Z3950_PREFIX,5,105,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_extended[] = {Z3950_PREFIX,5,106,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_fragment[] = {Z3950_PREFIX,5,107,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_pdf[] = {Z3950_PREFIX,5,109,1,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_postscript[] = {Z3950_PREFIX,5,109,2,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_html[] = {Z3950_PREFIX,5,109,3,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_tiff[] = {Z3950_PREFIX,5,109,4,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_gif[] = {Z3950_PREFIX,5,109,5,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_jpeg[] = {Z3950_PREFIX,5,109,6,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_png[] = {Z3950_PREFIX,5,109,7,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_mpeg[] = {Z3950_PREFIX,5,109,8,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_sgml[] = {Z3950_PREFIX,5,109,9,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_tiff_b[] = {Z3950_PREFIX,5,110,1,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_wav[] = {Z3950_PREFIX,5,110,2,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_sql_rs[] = {Z3950_PREFIX,5,111,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_soif[] = {Z3950_PREFIX,5,1000,81,2,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_json[] = {Z3950_PREFIX,5,1000,81,3,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_xml[] = {Z3950_PREFIX,5,109,10,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_text_xml[] = {Z3950_PREFIX,5,109,10,-1};
YAZ_EXPORT const Odr_oid yaz_oid_recsyn_application_xml[] = {Z3950_PREFIX,5,109,11,-1};
YAZ_EXPORT const Odr_oid yaz_oid_resform_resource_1[] = {Z3950_PREFIX,7,1,-1};
YAZ_EXPORT const Odr_oid yaz_oid_resform_resource_2[] = {Z3950_PREFIX,7,2,-1};
YAZ_EXPORT const Odr_oid yaz_oid_resform_universe_resource_report[] = {Z3950_PREFIX,7,1000,81,1,-1};
YAZ_EXPORT const Odr_oid yaz_oid_accform_prompt_1[] = {Z3950_PREFIX,8,1,-1};
YAZ_EXPORT const Odr_oid yaz_oid_accform_des_1[] = {Z3950_PREFIX,8,2,-1};
YAZ_EXPORT const Odr_oid yaz_oid_accform_krb_1[] = {Z3950_PREFIX,8,3,-1};
YAZ_EXPORT const Odr_oid yaz_oid_extserv_persistent_set[] = {Z3950_PREFIX,9,1,-1};
YAZ_EXPORT const Odr_oid yaz_oid_extserv_persistent_query[] = {Z3950_PREFIX,9,2,-1};
YAZ_EXPORT const Odr_oid yaz_oid_extserv_periodic_query[] = {Z3950_PREFIX,9,3,-1};
YAZ_EXPORT const Odr_oid yaz_oid_extserv_item_order[] = {Z3950_PREFIX,9,4,-1};
YAZ_EXPORT const Odr_oid yaz_oid_extserv_database_update_first_version[] = {Z3950_PREFIX,9,5,-1};
YAZ_EXPORT const Odr_oid yaz_oid_extserv_database_update_second_version[] = {Z3950_PREFIX,9,5,1,-1};
YAZ_EXPORT const Odr_oid yaz_oid_extserv_database_update[] = {Z3950_PREFIX,9,5,1,1,-1};
YAZ_EXPORT const Odr_oid yaz_oid_extserv_exp__spec_[] = {Z3950_PREFIX,9,6,-1};
YAZ_EXPORT const Odr_oid yaz_oid_extserv_exp__inv_[] = {Z3950_PREFIX,9,7,-1};
YAZ_EXPORT const Odr_oid yaz_oid_extserv_admin[] = {Z3950_PREFIX,9,1000,81,1,-1};
YAZ_EXPORT const Odr_oid yaz_oid_userinfo_searchresult_1[] = {Z3950_PREFIX,10,1,-1};
YAZ_EXPORT const Odr_oid yaz_oid_userinfo_charsetandlanguagenegotiation[] = {Z3950_PREFIX,10,2,-1};
YAZ_EXPORT const Odr_oid yaz_oid_userinfo_userinfo_1[] = {Z3950_PREFIX,10,3,-1};
YAZ_EXPORT const Odr_oid yaz_oid_userinfo_multiplesearchterms_1[] = {Z3950_PREFIX,10,4,-1};
YAZ_EXPORT const Odr_oid yaz_oid_userinfo_multiplesearchterms_2[] = {Z3950_PREFIX,10,5,-1};
YAZ_EXPORT const Odr_oid yaz_oid_userinfo_datetime[] = {Z3950_PREFIX,10,6,-1};
YAZ_EXPORT const Odr_oid yaz_oid_userinfo_proxy[] = {Z3950_PREFIX,10,1000,81,1,-1};
YAZ_EXPORT const Odr_oid yaz_oid_userinfo_cookie[] = {Z3950_PREFIX,10,1000,81,2,-1};
YAZ_EXPORT const Odr_oid yaz_oid_userinfo_client_ip[] = {Z3950_PREFIX,10,1000,81,3,-1};
YAZ_EXPORT const Odr_oid yaz_oid_userinfo_scan_set[] = {Z3950_PREFIX,10,1000,81,4,-1};
YAZ_EXPORT const Odr_oid yaz_oid_userinfo_facet_1[] = {Z3950_PREFIX,10,1000,81,5,-1};
YAZ_EXPORT const Odr_oid yaz_oid_elemspec_espec_1[] = {Z3950_PREFIX,11,1,-1};
YAZ_EXPORT const Odr_oid yaz_oid_varset_variant_1[] = {Z3950_PREFIX,12,1,-1};
YAZ_EXPORT const Odr_oid yaz_oid_schema_wais_schema[] = {Z3950_PREFIX,13,1,-1};
YAZ_EXPORT const Odr_oid yaz_oid_schema_gils_schema[] = {Z3950_PREFIX,13,2,-1};
YAZ_EXPORT const Odr_oid yaz_oid_schema_collections_schema[] = {Z3950_PREFIX,13,3,-1};
YAZ_EXPORT const Odr_oid yaz_oid_schema_geo_schema[] = {Z3950_PREFIX,13,4,-1};
YAZ_EXPORT const Odr_oid yaz_oid_schema_cimi_schema[] = {Z3950_PREFIX,13,5,-1};
YAZ_EXPORT const Odr_oid yaz_oid_schema_update_es[] = {Z3950_PREFIX,13,6,-1};
YAZ_EXPORT const Odr_oid yaz_oid_schema_holdings[] = {Z3950_PREFIX,13,7,-1};
YAZ_EXPORT const Odr_oid yaz_oid_schema_zthes[] = {Z3950_PREFIX,13,8,-1};
YAZ_EXPORT const Odr_oid yaz_oid_schema_thesaurus_schema[] = {Z3950_PREFIX,13,1000,81,1,-1};
YAZ_EXPORT const Odr_oid yaz_oid_schema_explain_schema[] = {Z3950_PREFIX,13,1000,81,2,-1};
YAZ_EXPORT const Odr_oid yaz_oid_tagset_tagsetm[] = {Z3950_PREFIX,14,1,-1};
YAZ_EXPORT const Odr_oid yaz_oid_tagset_tagsetg[] = {Z3950_PREFIX,14,2,-1};
YAZ_EXPORT const Odr_oid yaz_oid_tagset_stas_tagset[] = {Z3950_PREFIX,14,3,-1};
YAZ_EXPORT const Odr_oid yaz_oid_tagset_gils_tagset[] = {Z3950_PREFIX,14,4,-1};
YAZ_EXPORT const Odr_oid yaz_oid_tagset_collections_tagset[] = {Z3950_PREFIX,14,5,-1};
YAZ_EXPORT const Odr_oid yaz_oid_tagset_cimi_tagset[] = {Z3950_PREFIX,14,6,-1};
YAZ_EXPORT const Odr_oid yaz_oid_tagset_thesaurus_tagset[] = {Z3950_PREFIX,14,1000,81,1,-1};
YAZ_EXPORT const Odr_oid yaz_oid_tagset_explain_tagset[] = {Z3950_PREFIX,14,1000,81,2,-1};
YAZ_EXPORT const Odr_oid yaz_oid_tagset_zthes_tagset[] = {Z3950_PREFIX,14,8,-1};
YAZ_EXPORT const Odr_oid yaz_oid_negot_charset_3[] = {Z3950_PREFIX,15,3,-1};
YAZ_EXPORT const Odr_oid yaz_oid_negot_charset_4[] = {Z3950_PREFIX,15,4,-1};
YAZ_EXPORT const Odr_oid yaz_oid_negot_charset_id[] = {Z3950_PREFIX,15,1000,81,1,-1};
YAZ_EXPORT const Odr_oid yaz_oid_userinfo_cql[] = {Z3950_PREFIX,16,2,-1};
YAZ_EXPORT const Odr_oid yaz_oid_general_ucs_2[] = {1,0,10646,1,0,2,-1};
YAZ_EXPORT const Odr_oid yaz_oid_general_ucs_4[] = {1,0,10646,1,0,4,-1};
YAZ_EXPORT const Odr_oid yaz_oid_general_utf_16[] = {1,0,10646,1,0,5,-1};
YAZ_EXPORT const Odr_oid yaz_oid_general_utf_8[] = {1,0,10646,1,0,8,-1};
YAZ_EXPORT const Odr_oid yaz_oid_userinfo_oclc_userinfo[] = {Z3950_PREFIX,10,1000,17,1,-1};
YAZ_EXPORT const Odr_oid yaz_oid_extserv_xml_es[] = {Z3950_PREFIX,9,1000,105,4,-1};
YAZ_EXPORT struct yaz_oid_entry yaz_oid_standard_entries[] =
{
	{CLASS_TRANSYN, yaz_oid_transyn_ber, "BER"},
	{CLASS_TRANSYN, yaz_oid_transyn_iso2709, "ISO2709"},
	{CLASS_GENERAL, yaz_oid_general_isoill_1, "ISOILL-1"},
	{CLASS_ABSYN, yaz_oid_absyn_z_apdu, "Z-APDU"},
	{CLASS_APPCTX, yaz_oid_appctx_z_basic, "Z-BASIC"},
	{CLASS_ATTSET, yaz_oid_attset_bib_1, "Bib-1"},
	{CLASS_ATTSET, yaz_oid_attset_exp_1, "Exp-1"},
	{CLASS_ATTSET, yaz_oid_attset_ext_1, "Ext-1"},
	{CLASS_ATTSET, yaz_oid_attset_ccl_1, "CCL-1"},
	{CLASS_ATTSET, yaz_oid_attset_gils, "GILS"},
	{CLASS_ATTSET, yaz_oid_attset_gils_attset, "GILS-attset"},
	{CLASS_ATTSET, yaz_oid_attset_stas_attset, "STAS-attset"},
	{CLASS_ATTSET, yaz_oid_attset_collections_attset, "Collections-attset"},
	{CLASS_ATTSET, yaz_oid_attset_cimi_attset, "CIMI-attset"},
	{CLASS_ATTSET, yaz_oid_attset_geo_attset, "Geo-attset"},
	{CLASS_ATTSET, yaz_oid_attset_zbig, "ZBIG"},
	{CLASS_ATTSET, yaz_oid_attset_util, "Util"},
	{CLASS_ATTSET, yaz_oid_attset_xd_1, "XD-1"},
	{CLASS_ATTSET, yaz_oid_attset_zthes, "Zthes"},
	{CLASS_ATTSET, yaz_oid_attset_fin_1, "Fin-1"},
	{CLASS_ATTSET, yaz_oid_attset_dan_1, "Dan-1"},
	{CLASS_ATTSET, yaz_oid_attset_holdings, "Holdings"},
	{CLASS_ATTSET, yaz_oid_attset_marc, "MARC"},
	{CLASS_ATTSET, yaz_oid_attset_bib_2, "Bib-2"},
	{CLASS_ATTSET, yaz_oid_attset_zeerex, "ZeeRex"},
	{CLASS_ATTSET, yaz_oid_attset_thesaurus_attset, "Thesaurus-attset"},
	{CLASS_ATTSET, yaz_oid_attset_idxpath, "IDXPATH"},
	{CLASS_ATTSET, yaz_oid_attset_extlite, "EXTLITE"},
	{CLASS_DIAGSET, yaz_oid_diagset_bib_1, "Bib-1"},
	{CLASS_DIAGSET, yaz_oid_diagset_diag_1, "Diag-1"},
	{CLASS_DIAGSET, yaz_oid_diagset_diag_es, "Diag-ES"},
	{CLASS_DIAGSET, yaz_oid_diagset_diag_general, "Diag-General"},
	{CLASS_RECSYN, yaz_oid_recsyn_unimarc, "Unimarc"},
	{CLASS_RECSYN, yaz_oid_recsyn_intermarc, "Intermarc"},
	{CLASS_RECSYN, yaz_oid_recsyn_ccf, "CCF"},
	{CLASS_RECSYN, yaz_oid_recsyn_usmarc, "USmarc"},
	{CLASS_RECSYN, yaz_oid_recsyn_marc21, "MARC21"},
	{CLASS_RECSYN, yaz_oid_recsyn_ukmarc, "UKmarc"},
	{CLASS_RECSYN, yaz_oid_recsyn_normarc, "Normarc"},
	{CLASS_RECSYN, yaz_oid_recsyn_librismarc, "Librismarc"},
	{CLASS_RECSYN, yaz_oid_recsyn_danmarc, "Danmarc"},
	{CLASS_RECSYN, yaz_oid_recsyn_finmarc, "Finmarc"},
	{CLASS_RECSYN, yaz_oid_recsyn_mab, "MAB"},
	{CLASS_RECSYN, yaz_oid_recsyn_canmarc, "Canmarc"},
	{CLASS_RECSYN, yaz_oid_recsyn_sbn, "SBN"},
	{CLASS_RECSYN, yaz_oid_recsyn_picamarc, "Picamarc"},
	{CLASS_RECSYN, yaz_oid_recsyn_ausmarc, "Ausmarc"},
	{CLASS_RECSYN, yaz_oid_recsyn_ibermarc, "Ibermarc"},
	{CLASS_RECSYN, yaz_oid_recsyn_carmarc, "Carmarc"},
	{CLASS_RECSYN, yaz_oid_recsyn_malmarc, "Malmarc"},
	{CLASS_RECSYN, yaz_oid_recsyn_jpmarc, "JPmarc"},
	{CLASS_RECSYN, yaz_oid_recsyn_swemarc, "SWEmarc"},
	{CLASS_RECSYN, yaz_oid_recsyn_siglemarc, "SIGLEmarc"},
	{CLASS_RECSYN, yaz_oid_recsyn_isdsmarc, "ISDSmarc"},
	{CLASS_RECSYN, yaz_oid_recsyn_rusmarc, "RUSmarc"},
	{CLASS_RECSYN, yaz_oid_recsyn_hunmarc, "Hunmarc"},
	{CLASS_RECSYN, yaz_oid_recsyn_nacsis_catp, "NACSIS-CATP"},
	{CLASS_RECSYN, yaz_oid_recsyn_finmarc2000, "FINMARC2000"},
	{CLASS_RECSYN, yaz_oid_recsyn_marc21_fin, "MARC21-fin"},
	{CLASS_RECSYN, yaz_oid_recsyn_explain, "Explain"},
	{CLASS_RECSYN, yaz_oid_recsyn_sutrs, "SUTRS"},
	{CLASS_RECSYN, yaz_oid_recsyn_opac, "OPAC"},
	{CLASS_RECSYN, yaz_oid_recsyn_summary, "Summary"},
	{CLASS_RECSYN, yaz_oid_recsyn_grs_0, "GRS-0"},
	{CLASS_RECSYN, yaz_oid_recsyn_grs_1, "GRS-1"},
	{CLASS_RECSYN, yaz_oid_recsyn_extended, "Extended"},
	{CLASS_RECSYN, yaz_oid_recsyn_fragment, "Fragment"},
	{CLASS_RECSYN, yaz_oid_recsyn_pdf, "pdf"},
	{CLASS_RECSYN, yaz_oid_recsyn_postscript, "postscript"},
	{CLASS_RECSYN, yaz_oid_recsyn_html, "html"},
	{CLASS_RECSYN, yaz_oid_recsyn_tiff, "tiff"},
	{CLASS_RECSYN, yaz_oid_recsyn_gif, "gif"},
	{CLASS_RECSYN, yaz_oid_recsyn_jpeg, "jpeg"},
	{CLASS_RECSYN, yaz_oid_recsyn_png, "png"},
	{CLASS_RECSYN, yaz_oid_recsyn_mpeg, "mpeg"},
	{CLASS_RECSYN, yaz_oid_recsyn_sgml, "sgml"},
	{CLASS_RECSYN, yaz_oid_recsyn_tiff_b, "tiff-b"},
	{CLASS_RECSYN, yaz_oid_recsyn_wav, "wav"},
	{CLASS_RECSYN, yaz_oid_recsyn_sql_rs, "SQL-RS"},
	{CLASS_RECSYN, yaz_oid_recsyn_soif, "SOIF"},
	{CLASS_RECSYN, yaz_oid_recsyn_json, "JSON"},
	{CLASS_RECSYN, yaz_oid_recsyn_xml, "XML"},
	{CLASS_RECSYN, yaz_oid_recsyn_text_xml, "text-XML"},
	{CLASS_RECSYN, yaz_oid_recsyn_application_xml, "application-XML"},
	{CLASS_RESFORM, yaz_oid_resform_resource_1, "Resource-1"},
	{CLASS_RESFORM, yaz_oid_resform_resource_2, "Resource-2"},
	{CLASS_RESFORM, yaz_oid_resform_universe_resource_report, "UNIverse-Resource-Report"},
	{CLASS_ACCFORM, yaz_oid_accform_prompt_1, "Prompt-1"},
	{CLASS_ACCFORM, yaz_oid_accform_des_1, "Des-1"},
	{CLASS_ACCFORM, yaz_oid_accform_krb_1, "Krb-1"},
	{CLASS_EXTSERV, yaz_oid_extserv_persistent_set, "Persistent set"},
	{CLASS_EXTSERV, yaz_oid_extserv_persistent_query, "Persistent query"},
	{CLASS_EXTSERV, yaz_oid_extserv_periodic_query, "Periodic query"},
	{CLASS_EXTSERV, yaz_oid_extserv_item_order, "Item order"},
	{CLASS_EXTSERV, yaz_oid_extserv_database_update_first_version, "Database Update (first version)"},
	{CLASS_EXTSERV, yaz_oid_extserv_database_update_second_version, "Database Update (second version)"},
	{CLASS_EXTSERV, yaz_oid_extserv_database_update, "Database Update"},
	{CLASS_EXTSERV, yaz_oid_extserv_exp__spec_, "exp. spec."},
	{CLASS_EXTSERV, yaz_oid_extserv_exp__inv_, "exp. inv."},
	{CLASS_EXTSERV, yaz_oid_extserv_admin, "Admin"},
	{CLASS_USERINFO, yaz_oid_userinfo_searchresult_1, "searchResult-1"},
	{CLASS_USERINFO, yaz_oid_userinfo_charsetandlanguagenegotiation, "CharSetandLanguageNegotiation"},
	{CLASS_USERINFO, yaz_oid_userinfo_userinfo_1, "UserInfo-1"},
	{CLASS_USERINFO, yaz_oid_userinfo_multiplesearchterms_1, "MultipleSearchTerms-1"},
	{CLASS_USERINFO, yaz_oid_userinfo_multiplesearchterms_2, "MultipleSearchTerms-2"},
	{CLASS_USERINFO, yaz_oid_userinfo_datetime, "DateTime"},
	{CLASS_USERINFO, yaz_oid_userinfo_proxy, "Proxy"},
	{CLASS_USERINFO, yaz_oid_userinfo_cookie, "Cookie"},
	{CLASS_USERINFO, yaz_oid_userinfo_client_ip, "Client-IP"},
	{CLASS_USERINFO, yaz_oid_userinfo_scan_set, "Scan-Set"},
	{CLASS_USERINFO, yaz_oid_userinfo_facet_1, "Facet-1"},
	{CLASS_ELEMSPEC, yaz_oid_elemspec_espec_1, "Espec-1"},
	{CLASS_VARSET, yaz_oid_varset_variant_1, "Variant-1"},
	{CLASS_SCHEMA, yaz_oid_schema_wais_schema, "WAIS-schema"},
	{CLASS_SCHEMA, yaz_oid_schema_gils_schema, "GILS-schema"},
	{CLASS_SCHEMA, yaz_oid_schema_collections_schema, "Collections-schema"},
	{CLASS_SCHEMA, yaz_oid_schema_geo_schema, "Geo-schema"},
	{CLASS_SCHEMA, yaz_oid_schema_cimi_schema, "CIMI-schema"},
	{CLASS_SCHEMA, yaz_oid_schema_update_es, "Update ES"},
	{CLASS_SCHEMA, yaz_oid_schema_holdings, "Holdings"},
	{CLASS_SCHEMA, yaz_oid_schema_zthes, "Zthes"},
	{CLASS_SCHEMA, yaz_oid_schema_thesaurus_schema, "thesaurus-schema"},
	{CLASS_SCHEMA, yaz_oid_schema_explain_schema, "Explain-schema"},
	{CLASS_TAGSET, yaz_oid_tagset_tagsetm, "TagsetM"},
	{CLASS_TAGSET, yaz_oid_tagset_tagsetg, "TagsetG"},
	{CLASS_TAGSET, yaz_oid_tagset_stas_tagset, "STAS-tagset"},
	{CLASS_TAGSET, yaz_oid_tagset_gils_tagset, "GILS-tagset"},
	{CLASS_TAGSET, yaz_oid_tagset_collections_tagset, "Collections-tagset"},
	{CLASS_TAGSET, yaz_oid_tagset_cimi_tagset, "CIMI-tagset"},
	{CLASS_TAGSET, yaz_oid_tagset_thesaurus_tagset, "thesaurus-tagset"},
	{CLASS_TAGSET, yaz_oid_tagset_explain_tagset, "Explain-tagset"},
	{CLASS_TAGSET, yaz_oid_tagset_zthes_tagset, "Zthes-tagset"},
	{CLASS_NEGOT, yaz_oid_negot_charset_3, "Charset-3"},
	{CLASS_NEGOT, yaz_oid_negot_charset_4, "Charset-4"},
	{CLASS_NEGOT, yaz_oid_negot_charset_id, "Charset-ID"},
	{CLASS_USERINFO, yaz_oid_userinfo_cql, "CQL"},
	{CLASS_GENERAL, yaz_oid_general_ucs_2, "UCS-2"},
	{CLASS_GENERAL, yaz_oid_general_ucs_4, "UCS-4"},
	{CLASS_GENERAL, yaz_oid_general_utf_16, "UTF-16"},
	{CLASS_GENERAL, yaz_oid_general_utf_8, "UTF-8"},
	{CLASS_USERINFO, yaz_oid_userinfo_oclc_userinfo, "OCLC-userInfo"},
	{CLASS_EXTSERV, yaz_oid_extserv_xml_es, "XML-ES"},
	{CLASS_NOP, 0, 0}
};
